#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <fcntl.h>
#include <sys/mman.h>
#include "usersem.h"

int main(int argc, char *argv[])
{
	struct usersem *sem;
	int fd, usem_fd;
	void *map;

	if (argc != 4) {
		fprintf(stderr, "Usage: test <file> <offset> show|init|up|down|mem\n");
		exit(1);
	}
	fd = open(argv[1], O_RDWR);
	if (fd < 0) {
		perror("opening file");
		exit(1);
	}
	map = mmap(NULL, 4096, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
	if (map == MAP_FAILED) {
		perror("mmap");
		exit(1);
	}

	usem_fd = usem_open();
	if (usem_fd < 0) {
		perror("usem_open");
		exit(1);
	}
	sem = map + atoi(argv[2]);
	if (strcmp(argv[3], "init") == 0)
		usem_init(sem);
	else if (strcmp(argv[3], "up") == 0) {
		if (usem_up(sem, usem_fd) != 0) {
			perror("usem_up");
			exit(1);
		}
	} else if (strcmp(argv[3], "down") == 0) {
		if (usem_down(sem, usem_fd) != 0) {
			perror("usem_down");
			exit(1);
		}
	} else if (strcmp(argv[3], "mem") == 0) {
		struct usersem mysem;
		usem_init(&mysem);
		usem_down(&mysem, usem_fd);
		usem_down(&mysem, usem_fd);
	} else if (strcmp(argv[3], "show") == 0) {
		printf("count = %i, sleepers = %i\n",
		       sem->count, sem->num_sleepers);
	}
	usem_close(usem_fd);
	exit(0);
}
	
